# Copyright 2019 Objectif Libre
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
import os
from unittest import mock

from django import http

from cloudkittydashboard.tests import base


class PredictivePricingTest(base.TestCase):

    def setUp(self):
        super(PredictivePricingTest, self).setUp()

        os.environ['DJANGO_SETTINGS_MODULE'] = 'openstack_dashboard.settings'
        import horizon.tables  # noqa
        with mock.patch('horizon.tables'):
            from cloudkittydashboard.dashboards.project.rating.views \
                import quote
        os.environ.pop('DJANGO_SETTINGS_MODULE')
        self.quote = quote

    def _test_quote_request_not_ajax_post(self, arg):
        request = mock.MagicMock()
        if arg == 'ajax':
            request.is_ajax.return_value = False
        elif arg == 'method':
            request.method == 'POST'
        resp = self.quote(request)
        self.assertIsInstance(resp, http.HttpResponse)
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.content.decode(), '0.0')

    def test_quote_request_not_ajax(self):
        self._test_quote_request_not_ajax_post('ajax')

    def test_quote_request_wrong_method(self):
        self._test_quote_request_not_ajax_post('method')

    @mock.patch('cloudkittydashboard.dashboards.project.rating.views.api')
    def test_quote_does_update_request_dict(self, api_mock):
        body = [{'service': 'nope'}, {'other_key': None}]
        expected_body = [{u'service': 'test_service'},
                         {u'other_key': None, 'service': 'test_service'}]

        request = mock.MagicMock()
        request.is_ajax.return_value = True
        request.method = 'POST'
        request.body = json.dumps(body)

        client_mock = mock.MagicMock()
        client_mock.rating.get_quotation.return_value = 42.0
        api_mock.cloudkittyclient.return_value = client_mock

        settings = mock.MagicMock()
        settings.CLOUDKITTY_QUOTATION_SERVICE = 'test_service'
        with mock.patch(
                'cloudkittydashboard.dashboards.project.rating.views.settings',
                settings):
            resp = self.quote(request)

        api_mock.cloudkittyclient.assert_called_with(request)
        client_mock.rating.get_quotation.assert_called_with(
            res_data=expected_body)
        self.assertIsInstance(resp, http.HttpResponse)
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.content.decode(), '42.0')



# Copyright 2017 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from cloudkittydashboard.api import cloudkitty as api


class EditPriorityForm(forms.SelfHandlingForm):
    priority = forms.IntegerField(label=_("Priority"))

    def handle(self, request, data):
        ck_client = api.cloudkittyclient(request)
        try:
            priority = ck_client.rating.update_module(
                module_id=self.initial["module_id"], priority=data["priority"])
            messages.success(
                request,
                _('Successfully updated priority'))
            return priority
        except Exception:
            exceptions.handle(request,
                              _("Unable to update priority."))



# -*- coding: utf-8 -*-
# Copyright 2015 Objectif Libre
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from django.conf.urls import url  # noqa
from django.utils.translation import ugettext_lazy as _
from openstack_dashboard.dashboards.project.instances.workflows \
    import create_instance


# Instances Panel
class InstancePredictivePricing(create_instance.SetInstanceDetailsAction):
    class Meta(object):
        name = _("Details")
        help_text_template = ("project/rating/"
                              "_launch_details_price.html")

    def get_help_text(self, extra_context=None):
        extra = extra_context or {}
        extra['price'] = 0
        extra = super(InstancePredictivePricing,
                      self).get_help_text(extra_context=extra)
        return extra


create_instance.SetInstanceDetails.action_class = InstancePredictivePricing



# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tables


def get_details_link(datum):
    if datum.tenant_id:
        url = "horizon:admin:rating_summary:project_details"
        return reverse(url, kwargs={'project_id': datum.tenant_id})


class SummaryTable(tables.DataTable):
    project_id = tables.Column(
        'tenant_id', verbose_name=_("Project ID"), link=get_details_link)
    project_name = tables.Column(
        'name', verbose_name=_("Project Name"), link=get_details_link)
    total = tables.Column('rate', verbose_name=_("Project Total"))

    class Meta(object):
        name = "summary"
        verbose_name = _("Summary")


class TenantSummaryTable(tables.DataTable):
    res_type = tables.Column('res_type', verbose_name=_("Res Type"))
    rate = tables.Column('rate', verbose_name=_("Rate"))

    class Meta(object):
        name = "tenant_summary"
        verbose_name = _("Tenant Summary")



# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import ugettext_lazy as _
from horizon import tables

from openstack_dashboard.api import keystone as api_keystone

from cloudkittydashboard.api import cloudkitty as api
from cloudkittydashboard.dashboards.admin.summary import tables as sum_tables


class IndexView(tables.DataTableView):
    template_name = 'admin/rating_summary/index.html'
    table_class = sum_tables.SummaryTable

    def get_data(self):
        summary = api.cloudkittyclient(self.request).report.get_summary(
            groupby=['tenant_id'], all_tenants=True)['summary']

        tenants, _ = api_keystone.tenant_list(self.request)
        tenants = {tenant.id: tenant.name for tenant in tenants}
        summary.append({
            'tenant_id': 'ALL',
            'rate': sum([float(item['rate']) for item in summary]),
        })
        summary = api.identify(summary, key='tenant_id')
        for tenant in summary:
            tenant['name'] = tenants.get(tenant.id, '-')
        summary[-1]['name'] = 'Cloud Total'
        return summary


class TenantDetailsView(tables.DataTableView):
    template_name = 'admin/rating_summary/details.html'
    table_class = sum_tables.TenantSummaryTable
    page_title = _("Script Details : {{ table.project_id }}")

    def _get_cloud_total_summary(self):
        return api.cloudkittyclient(self.request).report.get_summary(
            groupby=['res_type'], all_tenants=True)['summary']

    def get_data(self):
        tenant_id = self.kwargs['project_id']
        if tenant_id == 'ALL':
            summary = self._get_cloud_total_summary()
        else:
            summary = api.cloudkittyclient(self.request).report.get_summary(
                groupby=['res_type'], tenant_id=tenant_id)['summary']

        summary.append({
            'tenant_id': tenant_id,
            'res_type': 'TOTAL',
            'rate': sum([float(item['rate']) for item in summary]),
        })
        summary = api.identify(summary, key='res_type', name=True)
        return summary
